require 'wiki/vikiwikistyle'

module Wiki
  class ExtendedVikiWikiStyle < VikiWikiStyle
    # <pre> soft verbatim
    FM_SOFTPRE_S    = '-{{{'
    FM_SOFTPRE_E    = '-}}}'
    RE_SOFTPRE_S    = /^-\{{3}$/
    RE_SOFTPRE_E    = /^-\}{3}$/

    # <span class="">
    FM_SPAN          = '@{%s %s}'
    RE_SPAN          = /^\@\{(#{TX_IDENT})\s+([^}]*)\}/

    BLOCK_PARSING_ORDER = [
      'plugin', 'div', 'softpre', 'pre', 'blockquote', 'comment',
      'h6', 'h5', 'h4', 'h3', 'h2',
      'hr', 'ul', 'ol', 'dl2', 'dl', 'table', 'table_n', 'p'
    ]

    INLINE_PARSING_ORDER = [
      'wikiname', 'url', 'span', 'inplugin', 'em', 'strong', 'del',
      'escape', 'aliasname', 'bracketname', 'img', 'a'
    ]

    def softpre_onparse(root, match, line, indent, lines)
      text = ''
      until lines.empty? do
        wline = lines.shift
        wsentence, windent = split_indent(wline)
        break if windent == indent and @my::RE_SOFTPRE_E === wsentence
        text << wline
      end
      npre = appendChild_block(root, 'pre', {'wiki-soft-verbatim' => 'yes'},
                               nil, indent)
      parse_inline(npre, text)
      return npre.parentNode
    end

    def span_onparse(root, match)
      appendChild_inline(root, 'span', {'class'=>match[1]}, match[2])
    end

    def pre_ontext(node, outobj='')
      case node.getAttribute('wiki-soft-verbatim')
      when 'yes'
        pre_s = @my::FM_SOFTPRE_S; pre_e = @my::FM_SOFTPRE_E
      else
        pre_s = @my::FM_PRE_S; pre_e = @my::FM_PRE_E
      end
      outobj << indent_line(node)
      outobj << text_format(pre_s)
      outobj << "\n"
      outobj << generateChild(node).chomp
      outobj << "\n"
      outobj << indent_line(node)
      outobj << text_format(pre_e)
      outobj << "\n"
      return outobj
    end

    def span_ontext(node, outobj='')
      klass = node.getAttribute('class')
      outobj << text_format(@my::FM_SPAN, klass, generateChild(node))
    end
  end
end
